@@ -1,4 +1,4 @@
-/*  pl-arith.c,v 1.9 1994/03/02 14:32:55 jan Exp
+/*  $Id: pl-arith.c,v 1.11 1994/10/19 15:14:51 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -292,7 +292,7 @@ isCurrentArithFunction(register FunctorDef f, register Module m)
   return r;
 }
 
-#if unix || EMX
+#if O_SIGNAL
 typedef void (*OsSigHandler)(int);
 
 static void
@@ -1021,7 +1021,7 @@ static struct arithFunction ar_functions[MAXARITHFUNCTIONS] = {
 void
 initArith(void)
 {
-#if unix || EMX
+#if O_SIGNAL
   pl_signal(SIGFPE, realExceptionHandler);
 #endif
 #if __TURBOC__
