@@ -18,6 +18,14 @@ If time is there I will have a look at all this to  clean  it.   Notably
 handling times must be cleaned, but that not only holds for this module.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#if defined(__WINDOWS__) || defined(__NT__)
+#include "windows.h"
+#undef TRANSPARENT
+#undef FD_SET
+#undef FD_ISSET
+#undef FD_ZERO
+#endif
+
 #include "pl-incl.h"
 #include "pl-ctype.h"
 #if unix || EMX
@@ -439,21 +447,38 @@ vPutf(char *fm, va_list args)
 }
 
 bool
-readLine(char *buf, int stream)
+readLine(char *buffer, int stream)
 { int oldin = Input;
   Char c;
   ttybuf tbuf;
+  char *buf = buffer;
 
   Input = stream;
   PushTty(&tbuf, TTY_RAW);		/* donot prompt! */
-  while( (c=Get0()) != EOF && c != '\n' && c != '\r' )
-    *buf++ = c;
+  for(;;)
+  { switch( (c=Get0()) )
+    { case '\n':
+      case '\r':
+      case EOF:
   PopTty(&tbuf);
 
   *buf++ = EOS;
   Input = oldin;
 
   return c == EOF ? FALSE : TRUE;
+      case '\b':
+	if ( buf > buffer )
+	{ buf--;
+	  Putf("\b \b");
+	}
+	break;
+      default:
+	buf[0] = c;
+	buf[1] = EOS;
+	Putf(buf);
+	buf++;
+    }
+  }
 }
 
 int
@@ -774,9 +799,9 @@ pl_wait_for_input(Word streams, Word available, Word timeout)
     to = NULL;
 
 #if hpux
-  select(max, (int*) &fds, NULL, NULL, to);
+  select(max+1, (int*) &fds, NULL, NULL, to);
 #else
-  select(max, &fds, NULL, NULL, to);
+  select(max+1, &fds, NULL, NULL, to);
 #endif
 
   for(n=0; n <= max; n++)
@@ -1541,13 +1566,13 @@ pl_chdir(Word dir)
 
   if ( s == (char *)NULL )
     return warning("chdir/1: instantiation fault");
-  if ( (s = ExpandOneFile(s)) == (char *)NULL )
+  if ( (s = ExpandOneFile(PrologPath(s))) == (char *)NULL )
     fail;
   
   if ( ChDir(s) )
     succeed;
 
-  return warning("chdir/1: cannot change directory to %s", s);
+  return warning("chdir/1: cannot change directory to %s: %s", s, OsError());
 }
 
 
