@@ -1,4 +1,4 @@
-/*  pl-gc.c,v 1.11 1994/03/02 14:33:07 jan Exp
+/*  $Id: pl-gc.c,v 1.13 1994/10/19 15:14:57 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -10,6 +10,11 @@
 /*#define O_DEBUG 1*/
 /*#define O_SECURE 1*/
 #include "pl-incl.h"
+#include <memory.h>
+
+#ifdef __sun__
+#define memmove(t, f, s) bcopy(t, f, s)
+#endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This module is based on
@@ -1279,9 +1284,14 @@ unlockw(Word p)
 
 void
 unlockp(Void ptr)
-{ lock l = *--pTop;
+{ Lock l = --pTop;
 
-  if ( ptr != lockAddress(l) )
+  DEBUG(5, printf("unlockp(&0x%x (0x%x)) at 0x%x\n",
+		  (unsigned long) ptr,
+		  (unsigned long) lockAddress(*l),
+		  (unsigned long) l));
+
+  if ( ptr != lockAddress(*l) )
     warning("Mismatch in lock()/unlock()\n");
 }
 
@@ -1794,7 +1804,6 @@ int l, g, t;
     long gsize = sizeStack(global);
     long tsize = sizeStack(trail);
     real time = CpuTime();
-    word key;
 
     DEBUG(0,
 	  printf("growStacks(0x%x, 0x%x, %c%c%c) l+g+t = %ld %ld %ld ...",
@@ -1811,7 +1820,7 @@ int l, g, t;
     SECURE(key = checkStacks(fr));
 
     if ( t )
-    { tsize = nextStackSize(&stacks.trail);
+    { tsize = nextStackSize((Stack) &stacks.trail);
       tb = realloc(tb, tsize);
       shift_status.trail_shifts++;
     }
@@ -1821,18 +1830,18 @@ int l, g, t;
       assert(lb == addPointer(gb, loffset));	
 
       if ( g )
-      { gsize = nextStackSize(&stacks.global);
+      { gsize = nextStackSize((Stack) &stacks.global);
 	shift_status.global_shifts++;
       }
       if ( l )
-      { lsize = nextStackSize(&stacks.local);
+      { lsize = nextStackSize((Stack) &stacks.local);
 	shift_status.local_shifts++;
       }
 
       gb = realloc(gb, lsize + gsize + GL_SEPARATION);
       lb = addPointer(gb, gsize + GL_SEPARATION);
       if ( g )				/* global enlarged; move local */
-	bcopy(addPointer(gb, loffset), lb, lsize);
+	memmove(addPointer(gb, loffset), lb, lsize);
     }
       
 #define PrintStackParms(stack, name, newbase, newsize) \
