@@ -1,4 +1,4 @@
-/*  pl-incl.h,v 1.26 1994/03/02 14:33:09 jan Exp
+/*  $Id: pl-incl.h,v 1.28 1994/10/19 15:14:59 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -23,6 +23,10 @@ appropriate machine description file.
 #include "md.h"
 #endif
 
+#ifdef __WATCOMC__
+#include "pl-nodef.h"
+#endif
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 		      PROLOG SYSTEM OPTIONS
 
@@ -83,6 +87,10 @@ handy for it someone wants to add a data type to the system.
 #endif
 #endif
 
+#if defined(unix) || defined(__unix__)
+#define O_SIGNAL 1
+#endif
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The macros below try to establish a common basis for various  compilers,
 so  we  can  write  most  of the real code without having to worry about
@@ -104,7 +112,10 @@ This has worked on TURBO_C not very long ago.
 #endif
 
 #include <stdio.h>
-#if unix || EMX
+#if O_XOS
+#include <xos/xos.h>
+#endif
+#if O_SIGNAL
 #include <sys/types.h>
 #include <signal.h>
 #endif
@@ -162,7 +173,7 @@ typedef char *			Void;
 typedef void *			Void;
 #endif
 
-#if unix || EMX
+#if O_SIGNAL
 #ifdef SIGNAL_HANDLER_TYPE
 typedef SIGNAL_HANDLER_TYPE (*handler_t)();
 #else
@@ -569,7 +580,7 @@ the range of integers to +- 2^25.  (Macros have to be rewritten))
 
 #if O_DATA_AT_0X4
 #define INDIRECT_MASK	0x20000000L	/* Indirect constant */
-#define INT_MASK	0x10000000L	/* Indirect constant */
+#define INT_MASK	0x10000000L	/* Integer constant */
 #define SIGN_MASK	0x40000000L     /* Sign of an integer */
 #define MASK_BITS	3		/* high order mask bits */
 #define SIGN_OFFSET     1               /* Offset of sign bit from M.S. bit */
@@ -588,6 +599,35 @@ the range of integers to +- 2^25.  (Macros have to be rewritten))
 #endif /* O_DATA_AT_0X2 */
 #endif /* O_DATA_AT_0X4 */
 
+#define makeRef(p)	((word)(-(long)(p)))
+#define unRef(w)	((Word)(-(long)(w)))
+#define isRef(w)	((long)(w) < 0)
+
+#if O_DATA_AT_0X8
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Masking schema for system which have their malloc()-area in the negative
+addresses (>= 0x80000000L).  We basically use  the same masking strategy
+as for low addresses (< 0x20000000L), but  we place the reference tag on
+0x10000000L.  NOT TESTED!!!
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#undef REF_MASK				/* redefine! */
+#undef MASK_BITS
+#undef REAL_MASK
+#undef makeRef
+#undef unRef
+#undef isRef
+
+#define REF_MASK	0x10000000L	/* put here */
+#define MASK_BITS	4
+#define REAL_MASK	0xe0000000L	/* real on indirect value */
+
+#define makeRef(p)	((word)(p) | REF_MASK)
+#define unRef(w)	((Word)((word)(w) & ~REF_MASK))
+#define isRef(w)	((word)(w) & REF_MASK)
+#endif /* O_DATA_AT_0X8 */
+
 #define LMASK_BITS	2		/* low order mask bits */
 #define DMASK_BITS	4		/* DATA_TAG_MASK bits */
 #define FIRST_MASK	0x00000002L	/* first member of relocation chain */
@@ -762,9 +802,6 @@ for a pointer.
 Handling references.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define makeRef(p)	((word)(-(long)(p)))
-#define unRef(w)	((Word)(-(long)(w)))
-#define isRef(w)	((long)(w) < 0)
 #define deRef(p)	{ while(isRef(*(p))) (p) = unRef(*(p)); }
 #define deRef2(p, d)	{ (d) = (p); deRef((d)); }
 
@@ -1320,7 +1357,7 @@ struct state
 
 GLOBAL State stateList;			/* list of loaded states */
 
-#if unix || EMX
+#if O_SIGNAL
 GLOBAL struct
 { handler_t os;				/* Os signal handler */
   handler_t user;			/* User signal handler */
