@@ -1,4 +1,4 @@
-/*  pl-prims.c,v 1.14 1994/03/02 14:33:20 jan Exp
+/*  $Id: pl-prims.c,v 1.15 1994/06/20 10:44:52 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -1433,11 +1433,11 @@ pl_home(Word h)
 */
 
 word
-pl_option(Word key, Word value)
+pl_option(Word key, Word old, Word new)
 { Atom result;
-  Atom k;
+  Atom k, n;
 
-  if (!isAtom(*key))
+  if ( !isAtom(*key) )
     fail;
   k = (Atom) *key;
 
@@ -1446,7 +1446,16 @@ pl_option(Word key, Word value)
   else if (k == ATOM_init_file) result = lookupAtom(options.initFile);
   else fail;
 
-  return unifyAtomic(value, result);
+  TRY(unifyAtomic(old, result));
+  
+  if ( !isAtom(*new) )
+    fail;
+  n = (Atom) *new;
+  if (     k == ATOM_goal)	options.goal     = stringAtom(n);
+  else if (k == ATOM_top_level) options.topLevel = stringAtom(n);
+  else				options.initFile = stringAtom(n);
+
+  succeed;
 }
 
 static bool
