@@ -1,4 +1,4 @@
-/*  pl-proc.c,v 1.12 1994/03/02 14:33:22 jan Exp
+/*  $Id: pl-proc.c,v 1.14 1994/10/19 15:15:09 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -369,7 +369,14 @@ unallocClause(Clause clause)
 void
 freeClause(Clause c)
 { if (c->XR_size)
-  { freeHeap(c->externals, sizeof(word) * c->XR_size);
+  { Word xr = c->externals;
+    int i;
+
+    for(i=c->XR_size; i>0; i--, xr++)
+    { if ( isReal(*xr) )
+	freeHeapReal(*xr);
+    }
+    freeHeap(c->externals, sizeof(word) * c->XR_size);
     statistics.externals -= c->XR_size;
   }
   statistics.codes -= c->code_size;
@@ -464,7 +471,7 @@ autoImport(FunctorDef f, Module m)
   p = isCurrentProcedure(f, m->super);	/* Link the two */
   if ( proc == NULL )			/* Create header if not there */
     proc = lookupProcedure(f, m);
-					/* save? */
+					/* safe? */
   freeHeap(proc->definition, sizeof(struct definition));
   proc->definition = p->definition;
 
