@@ -1,4 +1,4 @@
-/*  pl-read.c,v 1.16 1994/03/21 16:46:46 jan Exp
+/*  $Id: pl-read.c,v 1.18 1994/10/19 15:15:10 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -318,7 +318,7 @@ raw_read2(void)
     DEBUG(3, if ( Input == 0 ) printf("getchr() -> %d (%c)\n", c, c));
     DEBUG(3, if ( Input == 0 ) printf("here = %ld, base = %ld",
 				      (long) rb.here, (long) rb.base));
-#if !O_READLINE
+#if !O_READLINE && O_TERMIOS
     if ( c == ttytab.tab.c_cc[VEOF] )		/* little hack ... */
       c = EOF ;
 #endif
