@@ -1,4 +1,4 @@
-/*  pl-save.c,v 1.23 1994/03/02 14:33:25 jan Exp
+/*  $Id: pl-save.c,v 1.25 1994/10/19 15:15:11 jan Exp $
 
     Copyright (c) 1991 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -616,11 +616,20 @@ main(int argc, char **argv, char **env)
 #else /* !O_SAVE */
 
 int
+#ifdef __NT__
+ntmain(argc, argv, env)
+#else
 main(argc, argv, env)
+#endif
 int argc;
 char **argv;
 char **env;
-{ exit(startProlog(argc, argv, env));
+{
+#if O_HPFS
+  argv[0] = store_string(PrologPath(argv[0]));
+#endif
+
+  exit(startProlog(argc, argv, env));
   return 1;
 }
 
