@@ -1,4 +1,4 @@
-/*  pl-trace.c,v 1.10 1994/03/02 14:33:30 jan Exp
+/*  $Id: pl-trace.c,v 1.11 1994/06/20 10:44:56 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -310,7 +310,10 @@ traceAction(char *cmd, int port, LocalFrame frame, bool interactive)
 { int num_arg;				/* numeric argument */
   char *s;
 
-#define FeedBack(msg)	{ if (interactive) Putf(msg); }
+#define FeedBack(msg)	{ if (interactive) { if (cmd[1] != EOS) \
+					       Putf("\n"); \
+					     else \
+					       Putf(msg); } }
 #define Warn(msg)	{ if (interactive) Putf(msg); else warning(msg); }
 #define Default		(-1)
 
@@ -441,6 +444,9 @@ helpTrace(void)
   Putf("r:                 retry      s:                 skip\n");
   Putf("u:                 up         w:                 write\n");
   Putf("C:                 toggle show context\n");
+#if O_DEBUG
+  Putf("[level] D:	   set system debug level\n");
+#endif
 }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -759,7 +765,7 @@ again:
 void
 initTracer(void)
 { 
-#if unix || EMX
+#if O_SIGNAL
   pl_signal(SIGINT, interruptHandler);
 #endif
 
