@@ -1,4 +1,4 @@
-/*  pl-wic.c,v 1.12 1994/03/02 14:33:33 jan Exp
+/*  $Id: pl-wic.c,v 1.14 1994/10/19 15:15:13 jan Exp $
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -150,13 +150,14 @@ int arity;
 
   printf("%c\b", cur[n++ & 0x3]);
 }
-#endif
 
-#if unix || EMX
+#else /*!tos*/
+
 #define notifyLoad(file)
 #define notifyLoaded()
 #define notifyPredicate(name, arity)
-#endif
+
+#endif /* tos */
 
 static char *
 getString(FILE *fd)
@@ -713,7 +714,7 @@ openWic(char *file)
     exec = stringAtom(loaderstatus.orgsymbolfile);
   }
   DEBUG(1, printf("Executable = %s\n", exec));
-  if ( !(exec = OsPath(AbsoluteFile(PrologPath(exec)))) )
+  if ( !(exec = OsPath(AbsoluteFile(exec))) )
     fail;
   DEBUG(1, printf("Expanded executable = %s\n", exec));
 /*fprintf(wicFd, "#!%s -x\n", exec);*/
